# -*- coding: utf-8 -*-
# by einfall, pclin & Sven_H for DreamOS

from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Components.ActionMap import ActionMap
from Components.Sources.List import List
from Components.MenuList import MenuList
from Components.Label import Label
from Components.Sources.StaticText import StaticText
from Components.MultiContent import MultiContentEntryText, MultiContentEntryTextAlphaBlend
from Components.Pixmap import Pixmap
from Components.ConfigList import ConfigListScreen
from Components.config import config, getConfigListEntry, ConfigText, ConfigSelectionNumber, ConfigSubsection, NoSave
from enigma import gFont, ePoint, eSize, getDesktop, eListboxPythonMultiContent, RT_HALIGN_CENTER, RT_VALIGN_CENTER, RT_WRAP, eListbox, eTimer
from skin import TemplatedListFonts, parseColor
import subprocess
import threading, re, os, json
from Screens.Console import Console
from Components.FileList import FileList

sz_w = getDesktop(0).size().width()

hyperionremote_sh = "systemctl"
hyperioncontrol_version = "1.0"

json.encoder.FLOAT_REPR = lambda o: format(o, '.4f')

class hyperionControlColors(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlColors" position="center,100" size="900,900" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,820" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,820" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,820"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,820" size="200,70" scale="stretch"/>
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,820" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="680,820" size="200,70" text="Image Mode" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="10,800" size="880,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="850,780" transparent="1" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""

	else:
		skin = """
		<screen name="hyperionControlColors" position="center,80" size="590,620" transparent="0">
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="440,570" size="140,40" text="Image Mode" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="list" position="10,3" size="575,540" transparent="1" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	
	def __init__(self, session):
		Screen.__init__(self, session)
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'SetupActions',], {
		'cancel': self.key_exit,
		'ok'	: self.key_ok,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		'previousSection' : self.pageUp,
		'nextSection' : self.pageDown,
		})

		tlf = TemplatedListFonts()
		self.litems = [(('Maroon', '#800000'),), (('Dark Red', '#8B0000'),), (('Brown', '#A52A2A'),), (('Firebrick', '#B22222'),), (('Crimson', '#DC143C'),), (('Red', '#FF0000'),), (('Tomato', '#FF6347'),), (('Coral', '#FF7F50'),), (('Indian Red', '#CD5C5C'),), (('Light Coral', '#F08080'),), (('Dark Salmon', '#E9967A'),), (('Salmon', '#FA8072'),), (('Light Salmon', '#FFA07A'),), (('Orange Red', '#FF4500'),), (('Dark Orange', '#FF8C00'),), (('Orange', '#FFA500'),), (('Gold', '#FFD700'),), (('Dark Golden Rod', '#B8860B'),), (('Golden Rod', '#DAA520'),), (('Pale Golden rod', '#EEE8AA'),), (('Dark Khaki', '#BDB76B'),), (('Khaki', '#F0E68C'),), (('Olive', '#808000'),), (('Yellow', '#FFFF00'),), (('Yellow Green', '#9ACD32'),), (('Dark Olive Green', '#556B2F'),), (('Olive Drab', '#6B8E23'),), (('Lawn Green', '#7CFC00'),), (('Chart Reuse', '#7FFF00'),), (('Green Yellow', '#ADFF2F'),), (('Dark Green', '#006400'),), (('Green', '#008000'),), (('Forest Green', '#228B22'),), (('Lime', '#00FF00'),), (('Lime Green', '#32CD32'),), (('Light Green', '#90EE90'),), (('Pale Green', '#98FB98'),), (('Dark Sea Green', '#8FBC8F'),), (('Medium Spring Green', '#00FA9A'),), (('Spring Green', '#00FF7F'),), (('Sea Green', '#2E8B57'),), (('Medium Aqua Marine', '#66CDAA'),), (('Medium Sea Green', '#3CB371'),), (('Light Sea Green', '#20B2AA'),), (('Dark Slate Gray', '#2F4F4F'),), (('Teal', '#008080'),), (('Dark Cyan', '#008B8B'),), (('Cyan', '#00FFFF'),), (('Light Cyan', '#E0FFFF'),), (('Dark Turquoise', '#00CED1'),), (('Turquoise', '#40E0D0'),), (('Medium Turquoise', '#48D1CC'),), (('Pale Turquoise', '#AFEEEE'),), (('Aqua Marine', '#7FFFD4'),), (('Powder Blue', '#B0E0E6'),), (('Cadet Blue', '#5F9EA0'),), (('Steel Blue', '#4682B4'),), (('Corn Flower Blue', '#6495ED'),), (('Deep Sky Blue', '#00BFFF'),), (('Dodger Blue', '#1E90FF'),), (('Light Blue', '#ADD8E6'),), (('Sky Blue', '#87CEEB'),), (('Light Sky Blue', '#87CEFA'),), (('Midnight Blue', '#191970'),), (('Navy', '#000080'),), (('Dark Blue', '#00008B'),), (('Medium Blue', '#0000CD'),), (('Blue', '#0000FF'),), (('Royal Blue', '#4169E1'),), (('Blue Violet', '#8A2BE2'),), (('Indigo', '#4B0082'),), (('Dark Slate Blue', '#483D8B'),), (('Slate Blue', '#6A5ACD'),), (('Medium Slate Blue', '#7B68EE'),), (('Medium Purple', '#9370DB'),), (('Dark Magenta', '#8B008B'),), (('Dark Violet', '#9400D3'),), (('Dark Orchid', '#9932CC'),), (('Medium Orchid', '#BA55D3'),), (('Purple', '#800080'),), (('Thistle', '#D8BFD8'),), (('Plum', '#DDA0DD'),), (('Violet', '#EE82EE'),), (('Magenta / Fuchsia', '#FF00FF'),), (('Orchid', '#DA70D6'),), (('Medium Violet Red', '#C71585'),), (('Pale Violet Red', '#DB7093'),), (('Deep Pink', '#FF1493'),), (('Hot Pink', '#FF69B4'),), (('Light Pink', '#FFB6C1'),), (('Pink', '#FFC0CB'),), (('Antique White', '#FAEBD7'),), (('Beige', '#F5F5DC'),), (('Bisque', '#FFE4C4'),), (('Blanched Almond', '#FFEBCD'),), (('Wheat', '#F5DEB3'),), (('Corn Silk', '#FFF8DC'),), (('Lemon Chiffon', '#FFFACD'),), (('Light Golden Rod Yellow', '#FAFAD2'),), (('Light Yellow', '#FFFFE0'),), (('Saddle Brown', '#8B4513'),), (('Sienna', '#A0522D'),), (('Chocolate', '#D2691E'),), (('Peru', '#CD853F'),), (('Sandy Brown', '#F4A460'),), (('Burly Wood', '#DEB887'),), (('Tan', '#D2B48C'),), (('Rosy Brown', '#BC8F8F'),), (('Moccasin', '#FFE4B5'),), (('Navajo White', '#FFDEAD'),), (('Peach Puff', '#FFDAB9'),), (('Misty Rose', '#FFE4E1'),), (('Lavender Blush', '#FFF0F5'),), (('Linen', '#FAF0E6'),), (('Old Lace', '#FDF5E6'),), (('Papaya Whip', '#FFEFD5'),), (('Sea Shell', '#FFF5EE'),), (('Mint Cream', '#F5FFFA'),), (('Slate Gray', '#708090'),), (('Light Slate Gray', '#778899'),), (('Light Steel Blue', '#B0C4DE'),), (('Lavender', '#E6E6FA'),), (('Floral White', '#FFFAF0'),), (('Alice Blue', '#F0F8FF'),), (('Ghost White', '#F8F8FF'),), (('Honeydew', '#F0FFF0'),), (('Ivory', '#FFFFF0'),), (('Azure', '#F0FFFF'),), (('Snow', '#FFFAFA'),), (('Dim Gray', '#696969'),), (('Gray', '#808080'),), (('Dark gray', '#A9A9A9'),), (('Silver', '#C0C0C0'),), (('Light Gray', '#D3D3D3'),), (('Gainsboro', '#DCDCDC'),), (('White Smoke', '#F5F5F5'),), (('White', '#FFFFFF'),)]
		
		if sz_w == 1920:
			self.itemHeight = 60
			self.itemWidth = 850
			self.ml = MenuList(self.litems, content=eListboxPythonMultiContent, itemSize=self.itemHeight, margin=ePoint(0,0), selectionZoom=1.1)
		else:
			self.itemHeight = 45
			self.itemWidth = 575
			self.ml = MenuList(self.litems, content=eListboxPythonMultiContent, itemSize=self.itemHeight, margin=ePoint(0,0), selectionZoom=1.1)
		self.ml.l.setFont(0, gFont(tlf.face(tlf.SMALL), tlf.size(tlf.SMALL)))
		self.ml.l.setFont(1, gFont(tlf.face(tlf.MEDIUM), tlf.size(tlf.MEDIUM)))
		self.ml.l.setBuildFunc(self.showList, True)
		self['list'] = self.ml
		self.onLayoutFinish.append(self.buildList)
		self.setTitle("Hyperion Control - Colors")
		
	def buildList(self):
		self["list"].setList(self.litems)
		try:
			import hyperion
			print "== hyperion.args", hyperion.args
		except:
			print "== error on load hyperion.args"
			import traceback
			traceback.print_exc()

	def pageUp(self):
		self["list"].pageUp()

	def pageDown(self):
		self["list"].pageDown()
		
	def showList(self, item, selected):
		(name, hcol) = item
		col = parseColor(hcol).argb()
		if selected:
			textcol = parseColor("#000000").argb()
			if col == textcol:
				textcol = parseColor("#FFFFFF").argb()
		else:
			textcol = parseColor("#FFFFFF").argb()
			if col == textcol:
				textcol = parseColor("#000000").argb()
		text = name
		res = [item]
		if sz_w == 1920:
			margin = 5
			pos = (margin+20,margin)
			EntrySize = (self.itemWidth - margin*2 -30,self.itemHeight-margin*2)
		else:
			margin = 4
			pos = (margin+15,margin)
			EntrySize = (self.itemWidth - margin*2 -20,self.itemHeight-margin*2)
		try:
			if selected:
				res.append(MultiContentEntryTextAlphaBlend(pos=pos, size=EntrySize, font=1, text=text, flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER|RT_WRAP, color=textcol, color_sel=textcol, backcolor=col, backcolor_sel=col))
			else:
				res.append(MultiContentEntryTextAlphaBlend(pos=pos, size=EntrySize, font=0, text=text, flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER|RT_WRAP, color=textcol, color_sel=textcol, backcolor=col, backcolor_sel=col))
			return res
		except:
			import traceback
			traceback.print_exc()

			return None

	def key_red(self):
		self.key_exit()

	def key_green(self):
		pass
		
	def key_yellow(self):
		pass

	def key_blue(self):
		try:
			#path = os.path.dirname(lastpicture) + "/"
			if config.plugins.hyperioncontrol.lastpicpath.value:
				path = config.plugins.hyperioncontrol.lastpicpath.value
			else:
				path = "/"
			print "basename", path
			self.session.openWithCallback(self.selectPictureCallback, HyperionControlPictureBrowser, path)
		except:
			import traceback
			traceback.print_exc()

	def selectPictureCallback(self, retvalue):
		if retvalue:
			cmd = "hyperion-remote --clearall && hyperion-remote -i " + retvalue
			print "[HyperionControl] cmd", cmd
			command = Command(cmd)
			command.run(timeout=3)
			print "[HyperionControl] cmd_out:", command.out

	def key_ok(self):
		exist = self['list'].getCurrent()
		if exist == None:
			return
		(name, color) = self['list'].getCurrent()[0]
		print name, color
		cmd = "-c '%s'" % color
		send_CMD(cmd)

	def key_exit(self):
		self.close()
		

#to get consolen-output with timeout-function
class Command(object):
	def __init__(self, cmd):
		self.cmd = cmd
		self.process = None

	def run(self, timeout):
		def target():
			self.process = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
			(self.out,self.err) = self.process.communicate()

		thread = threading.Thread(target=target)
		thread.start()
		self.timeout=False

		thread.join(float(timeout))
		if thread.is_alive():
			self.process.terminate()
			self.timeout=True
			self.process = None
			thread = None
			return
#=============================================

class hyperionControlSetup(Screen, ConfigListScreen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlSetup" position="center,100" size="900,900" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,820" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,820" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,820"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,820" size="200,70" scale="stretch"/>
			<eLabel name="" position="10,800" size="880,2" backgroundColor="grey" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,820" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="240,820" size="200,70" text="save config" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="460,820" size="200,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;30" foregroundColor="white" position="680,820" size="200,70" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="config" seperation="100" position="10,10" size="880,770" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionControlSetup" position="center,80" size="590,620">
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="150,570" size="140,40" text="save config" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<widget source="key_yellow" render="Label" font="Regular;22" foregroundColor="white" position="295,570" size="140,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;22" foregroundColor="white" position="440,570" size="140,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="config" position="10,10" seperation="100" size="570,545" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
		
	def __init__(self, session):
		Screen.__init__(self, session)
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions'], {
		'cancel': self.key_exit,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		})
		
		self["key_yellow"] = StaticText()
		self["key_blue"] = StaticText("import bak")
		self.setupPage = "transform"
		
		self.list = []
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changed)
		self.buildConfig()
		self.setTitle("Hyperion Control - Setup")
		self["key_yellow"].setText("Device/LED")
		self.onShown.append(self.readData)
		
	def readData(self):
		self.onShown.remove(self.readData)
		self.readValuesFromConfig()
		readValuesFromRemote(self)
	
	def saveJson(self):
		try:
			# hsv
			self.jsonConfig['color']['transform'][0]['hsv']['saturationGain'] = int(config.plugins.hyperioncontrol.saturationGain.value) / 100.0
			self.jsonConfig['color']['transform'][0]['hsv']['valueGain'] = int(config.plugins.hyperioncontrol.valueGain.value) / 100.0
			
			# hsl
			self.jsonConfig['color']['transform'][0]['hsl']['saturationLGain'] = int(config.plugins.hyperioncontrol.saturationLGain.value) / 100.0
			self.jsonConfig['color']['transform'][0]['hsl']['luminanceGain'] = int(config.plugins.hyperioncontrol.luminanceGain.value) / 100.0
			self.jsonConfig['color']['transform'][0]['hsl']['luminanceMinimum'] = int(config.plugins.hyperioncontrol.luminanceMinimum.value) / 100.0
			
			# red
			self.jsonConfig['color']['transform'][0]['red']['gamma'] = int(config.plugins.hyperioncontrol.gammaRed.value) / 100.0
			self.jsonConfig['color']['transform'][0]['red']['threshold'] = int(config.plugins.hyperioncontrol.thresholdRed.value) / 100.0
			self.jsonConfig['color']['temperature'][0]['correctionValues']['red'] = int(config.plugins.hyperioncontrol.temperatureRed.value) 
			self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['redChannel'] = int(config.plugins.hyperioncontrol.redAdjustRed.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['redChannel'] = int(config.plugins.hyperioncontrol.greenAdjustRed.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['redChannel'] = int(config.plugins.hyperioncontrol.blueAdjustRed.value)
			self.jsonConfig['color']['transform'][0]['red']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelRed.value) / 100.0
			self.jsonConfig['color']['transform'][0]['red']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelRed.value) / 100.0
			
			# green
			self.jsonConfig['color']['transform'][0]['green']['gamma'] = int(config.plugins.hyperioncontrol.gammaGreen.value) / 100.0
			self.jsonConfig['color']['transform'][0]['green']['threshold'] = int(config.plugins.hyperioncontrol.thresholdGreen.value) / 100.0
			self.jsonConfig['color']['temperature'][0]['correctionValues']['green'] = int(config.plugins.hyperioncontrol.temperatureGreen.value) 
			self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['greenChannel'] = int(config.plugins.hyperioncontrol.redAdjustGreen.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['greenChannel'] = int(config.plugins.hyperioncontrol.greenAdjustGreen.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['greenChannel'] = int(config.plugins.hyperioncontrol.blueAdjustGreen.value)
			self.jsonConfig['color']['transform'][0]['green']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelGreen.value) / 100.0
			self.jsonConfig['color']['transform'][0]['green']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelGreen.value) / 100.0
			
			# blue
			self.jsonConfig['color']['transform'][0]['blue']['gamma'] = int(config.plugins.hyperioncontrol.gammaBlue.value) / 100.0
			self.jsonConfig['color']['transform'][0]['blue']['threshold'] = int(config.plugins.hyperioncontrol.thresholdBlue.value) / 100.0
			self.jsonConfig['color']['temperature'][0]['correctionValues']['blue'] = int(config.plugins.hyperioncontrol.temperatureBlue.value) 
			self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['blueChannel'] = int(config.plugins.hyperioncontrol.redAdjustBlue.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['blueChannel'] = int(config.plugins.hyperioncontrol.greenAdjustBlue.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['blueChannel'] = int(config.plugins.hyperioncontrol.blueAdjustBlue.value)
			self.jsonConfig['color']['transform'][0]['blue']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelBlue.value) / 100.0
			self.jsonConfig['color']['transform'][0]['blue']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelBlue.value) / 100.0
			
			#Framegrabber
			self.jsonConfig['framegrabber']['width'] = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[0])
			self.jsonConfig['framegrabber']['height'] = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[1])
			self.jsonConfig['framegrabber']['frequency_Hz'] = int(config.plugins.hyperioncontrol.framegrabberFreq.value) * 1.0
			
			#BlackborderDetector
			self.jsonConfig['blackborderdetector']['enable'] = str(config.plugins.hyperioncontrol.bbdEnable.value).lower()
			self.jsonConfig['blackborderdetector']['threshold'] = int(config.plugins.hyperioncontrol.bbdThreshold.value) / 100.0
			
			# device
			self.jsonConfig['device']['type'] = str(config.plugins.hyperioncontrol.deviceType.value)
			self.jsonConfig['device']['name'] = str(config.plugins.hyperioncontrol.configName.value)
			self.jsonConfig['device']['output'] = ".".join(["%d" % d for d in config.plugins.hyperioncontrol.outputIP.value]) + ":19446"
			self.jsonConfig['device']['colorOrder'] = str(config.plugins.hyperioncontrol.colorOrder.value)
			self.jsonConfig['device']['rate'] = int(config.plugins.hyperioncontrol.baudrate.value)
			
			#LED-Config
			if config.plugins.hyperioncontrol.ledBegin.value != "0":
				LEDconfig = self.createLEDConfig()
				if LEDconfig:
					self.jsonConfig['leds'] = LEDconfig
				else:
					self.session.open(MessageBox,"Fehler beim Erzeugen der LED-Config!", MessageBox.TYPE_INFO,timeout=5)
			
			#aml-Service
			self.writeAmlService()
			
			config_file_path ="/etc/hyperion/hyperion.config.json"
			
			if os.path.exists(config_file_path): 
				#write values in config.json
				with open(config_file_path, "w") as write_file:
					json.dump(self.jsonConfig, write_file, ensure_ascii=False, indent=4)
				#daemon reload und services neu starten
				cmd = "systemctl daemon-reload && systemctl restart hyperion && systemctl restart hyperionaml"
				os.system(cmd)
				self.session.open(MessageBox,"Die Einstellungen wurden in '/etc/hyperion/hyperion.config.json' gespeichert.\n\nHyperion und Hyperionaml wurden neu gestartet.", MessageBox.TYPE_INFO,timeout=10)
			else:
				self.session.open(MessageBox,"Keine /etc/hyperion/hyperion.config.json gefunden!", MessageBox.TYPE_INFO,timeout=5)
			
			config.plugins.hyperioncontrol.save()
			
			
		
		except:
			import traceback
			traceback.print_exc()
			self.session.open(MessageBox,"Beim Speichern der config.json ist ein Fehler aufgetreten!", MessageBox.TYPE_INFO,timeout=5)

	def writeAmlService(self):
		
		script_txt = "[Unit]\n"
		script_txt += "Description=Hyperionaml Systemd service\n"
		script_txt += "Wants=enigma2.service\n"
		script_txt += "Wants=hyperion.service\n"
		script_txt += "[Service]\n"
		script_txt += "Type=simple\n"
		script_txt += "User=root\n"
		script_txt += "Group=root\n"
		script_txt += "UMask=007\n"
		
		width = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[0])
		height = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[1])
		freq = config.plugins.hyperioncontrol.framegrabberFreq.value
		script_txt += "ExecStart=/usr/bin/hyperion-aml -f %s --width %s --height %s\n" % (freq, width, height)
		
		script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
		script_txt += "Restart=on-failure\n"
		script_txt += "TimeoutStopSec=10\n"
		script_txt += "[Install]\n"
		script_txt += "WantedBy=multi-user.target\n"
		
		amlservice_file = open("/lib/systemd/system/hyperionaml.service", "w+")
		amlservice_file.write(script_txt)
		amlservice_file.close()
		os.chmod("/lib/systemd/system/hyperionaml.service",0644)
		
	def getFixScanValue(self, ledNr, changes):
		#print "== scanValue", ledNr, changes
		hPicDeep = int(config.plugins.hyperioncontrol.hPicDeep.value) / 100.0
		vPicDeep = int(config.plugins.hyperioncontrol.vPicDeep.value) / 100.0
		
		if ledNr <= changes[0]:
			return (1.000 - hPicDeep, 1.0000) #vscan bottom
		elif ledNr <= changes[1]:
			if config.plugins.hyperioncontrol.ledDirection.value == "1":
				return (0.0000, vPicDeep) #hscan left
			else:
				return (1.0000 - vPicDeep, 1.0000) #hscan right
		elif ledNr <= changes[2]:
			return (0.0000, hPicDeep) #vscan top
		elif ledNr <= changes[3]:
			if config.plugins.hyperioncontrol.ledDirection.value == "1":
				return (1.0000 - vPicDeep, 1.0000) #hscan right
			else:
				return (0.0000, vPicDeep) #hscan left
		else:
			return (1.0000 - hPicDeep, 1.0000) #vscan bottom

	def createLEDConfig(self):
		try:
			
			led_count = int(config.plugins.hyperioncontrol.ledTop.value)*2 + int(config.plugins.hyperioncontrol.ledRight.value) + int(config.plugins.hyperioncontrol.ledLeft.value) - int(config.plugins.hyperioncontrol.ledBottomGap.value)
			#print "=== led-count", led_count
			
			h_diff = 1.0 / int(config.plugins.hyperioncontrol.ledTop.value)
			vr_diff = 1.0 / int(config.plugins.hyperioncontrol.ledRight.value)
			vl_diff = 1.0 / int(config.plugins.hyperioncontrol.ledLeft.value)
			gap_diff = int(config.plugins.hyperioncontrol.ledBottomGap.value) // 2
			
			if config.plugins.hyperioncontrol.ledDirection.value == "1":
				h_diff = h_diff * -1
				vr_diff = vr_diff * -1
				vl_diff = vl_diff * -1
				
			#print "=== vr_diff", vr_diff
			
			#wenn ledBegin "unten rechts"
			if config.plugins.hyperioncontrol.ledBegin.value == "2":
				#todo use gap_diff
				if config.plugins.hyperioncontrol.ledDirection.value == "1": # UZ
					change0 = int(config.plugins.hyperioncontrol.ledTop.value)
					change1 = change0 + int(config.plugins.hyperioncontrol.ledLeft.value)
					change2 = change1 + int(config.plugins.hyperioncontrol.ledTop.value)
					change3 = change2 + int(config.plugins.hyperioncontrol.ledRight.value)
					positions = ["bottom","left","top","right"]
				else: #GUZ
					change0 = int(config.plugins.hyperioncontrol.ledRight.value)
					change1 = change0 + int(config.plugins.hyperioncontrol.ledTop.value)
					change2 = change1 + int(config.plugins.hyperioncontrol.ledLeft.value)
					change3 = change2 + int(config.plugins.hyperioncontrol.ledTop.value)
					positions = ["right","top","left","bottom"]
				changes = [change0,change1,change2,change3]
			
			# wenn ledBegin "unten Mitte"
			if config.plugins.hyperioncontrol.ledBegin.value == "1":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": # UZ
					change0 = int(config.plugins.hyperioncontrol.ledTop.value)//2 - gap_diff
					change1 = change0 + int(config.plugins.hyperioncontrol.ledLeft.value)
					change2 = change1 + int(config.plugins.hyperioncontrol.ledTop.value)
					change3 = change2 + int(config.plugins.hyperioncontrol.ledRight.value)
					positions = ["bottom","left","top","right","bottom"]
				else: #GUZ
					change0 = int(config.plugins.hyperioncontrol.ledTop.value)//2 - gap_diff
					change1 = change0 + int(config.plugins.hyperioncontrol.ledRight.value)
					change2 = change1 + int(config.plugins.hyperioncontrol.ledTop.value)
					change3 = change2 + int(config.plugins.hyperioncontrol.ledLeft.value)
					positions = ["bottom","right","top","left","bottom"]
				changes = [change0,change1,change2,change3]
				
				#print "=== changes", changes
				leds = []
				for i in range(1,led_count+1):
					#unten ab Mitte
					if i <= changes[0]:
						if i ==1:
							hscan_min = 0.5 + (h_diff * gap_diff)
							if config.plugins.hyperioncontrol.ledDirection.value == "2": #bei GUZ
								hscan_min -= h_diff
						hscan_min = hscan_min + h_diff
						hscan_max = hscan_min + abs(h_diff)
						(vscan_min, vscan_max) = self.getFixScanValue(i,changes)
					#rechts
					elif i <= changes[1]:
						(hscan_min, hscan_max) = self.getFixScanValue(i,changes)
						if i == changes[0]+1: # bei GUZ Wechsel von unten auf rechts
							vscan_min = 1.0000
						vscan_min = vscan_min - abs(vr_diff)
						vscan_max = vscan_min + abs(vr_diff)
					#oben
					elif i <= changes[2]:
						if i == changes[1]+1: # bei GUZ Wechsel rechts auf oben
							hscan_min = 1.0000
							if config.plugins.hyperioncontrol.ledDirection.value == "1": # bei UZ links auf oben
								hscan_min = 0.0000 - abs(h_diff)
						hscan_min = hscan_min - h_diff
						hscan_max = hscan_min + abs(h_diff)
						(vscan_min, vscan_max) = self.getFixScanValue(i,changes)
					#links
					elif i <= changes[3]:
						(hscan_min, hscan_max) = self.getFixScanValue(i,changes)
						if i == changes[2]+1: # bei Wechsel auf links
							vscan_min = 0.0000 - abs(vl_diff)
						vscan_min = vscan_min + abs(vl_diff)
						vscan_max = vscan_min + abs(vl_diff)
					#unten bis Mitte
					else:
						if i == changes[3]+1: # bei GUZ Wechsel von links auf unten
							hscan_min = 0.0000 - h_diff
							if config.plugins.hyperioncontrol.ledDirection.value == "1": # bei UZ rechts auf unten
								hscan_min = 1.0000
						hscan_min = hscan_min + h_diff
						hscan_max = hscan_min + abs(h_diff)
						(vscan_min, vscan_max) = self.getFixScanValue(i,changes)
					
					from collections import OrderedDict
					led = OrderedDict()
					led['index'] = int(i-1)
					led['hscan'] = {'minimum': abs(hscan_min), 'maximum': abs(hscan_max)}
					led['vscan'] = {'minimum': abs(vscan_min), 'maximum': abs(vscan_max)}
					#leds.append({'index': int(i-1), 'hscan': {'minimum': hscan_min, 'maximum': hscan_max}, 'vscan': {'minimum': vscan_min, 'maximum': vscan_max}})
					leds.append(led)
					#print "=== led dict", led
					
					#print "index: ", i-1
					#print '"hscan" : { "minimum" : %s, "maximum" : %s },' % (round(hscan_min,4), round(hscan_max,4))
					#print '"vscan" : { "minimum" : %s, "maximum" : %s }' % (vscan_min, vscan_max)
					#print "------------------------------------------------"
				#print "=== leds", leds
				return leds
			else:
				self.session.open(MessageBox,"Die LED-Konfiguration fr diesen 'Anfang' wird beim Speichern noch nicht untersttzt!", MessageBox.TYPE_INFO,timeout=5)
				return None
		except:
			import traceback
			traceback.print_exc()
			#self.session.open(MessageBox,"Fehler beim Erzeugen der LED-Config!!", MessageBox.TYPE_INFO,timeout=5)
			return None

	def importJsonBak(self):
		self.session.openWithCallback(self.importJsonBakCallBack, MessageBox,"Sollen alle Daten aus einer existieren hyperion.config.json.bak importiert werden?\n\nAlle Setup-Werte werden mit den Daten aus der hyperion.config.json.bak berschrieben!\n\nDie eingelesenen Werte koennen anschliessend ber die grne Taste in der aktuellen hyperion.config.json gespeichert werden.", MessageBox.TYPE_YESNO,default=False)
	
	def importJsonBakCallBack(self, retval):
		try:
			if not retval:
				return
			
			#read values from config.json.bak
			if self.jsonConfig == None:
				self.session.open(MessageBox,"Es liegen keine Daten aus /etc/hyperion/hyperion.config.json vor!\n\nEin Import ist daher nicht mglich.", MessageBox.TYPE_INFO,timeout=5)
				return
			
			self.jsonConfigBak= ""
			config_file_path ="/etc/hyperion/hyperion.config.json.bak"
			if os.path.exists(config_file_path): 
				#json.bak einlesen
				valuesFromConfig = open(config_file_path).read()
				lines = valuesFromConfig.split("\n")
				newlines=[]
				for line in lines:
					cleanline = line.replace("\t","")
					if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
						newlines.append(line)
				valuesFromConfig = "\n".join(newlines)
				from collections import OrderedDict
				self.jsonConfigBak = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)
				
				#alle Hauptkeys aus json.bak in json bernehmen
				for (k, v) in self.jsonConfigBak.items():
					#print "===", k
					key = str(k)
					if key in self.jsonConfig:
						self.jsonConfig[key] = self.jsonConfigBak[key]
				
				#reload configs from jsonConfigBak
				# hsv
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationGain, self.jsonConfigBak, ['color','transform',0,'hsv','saturationGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.valueGain, self.jsonConfigBak, ['color','transform',0,'hsv','valueGain'],100)
				# hsl
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationLGain, self.jsonConfigBak, ['color','transform',0,'hsl','saturationLGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceGain, self.jsonConfigBak, ['color','transform',0,'hsl','luminanceGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceMinimum, self.jsonConfigBak, ['color','transform',0,'hsl','luminanceMinimum'],100)
				# red
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaRed, self.jsonConfigBak, ['color','transform',0,'red','gamma'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdRed, self.jsonConfigBak, ['color','transform',0,'red','threshold'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureRed, self.jsonConfigBak, ['color','temperature',0,'correctionValues','red'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustRed, self.jsonConfigBak, ['color','channelAdjustment',0,'pureRed','redChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustRed, self.jsonConfigBak, ['color','channelAdjustment',0,'pureGreen','redChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustRed, self.jsonConfigBak, ['color','channelAdjustment',0,'pureBlue','redChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelRed, self.jsonConfigBak, ['color','transform',0,'red','whitelevel'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelRed, self.jsonConfigBak, ['color','transform',0,'red','blacklevel'],100)
				
				# green
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaGreen, self.jsonConfigBak, ['color','transform',0,'green','gamma'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdGreen, self.jsonConfigBak, ['color','transform',0,'green','threshold'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureGreen, self.jsonConfigBak, ['color','temperature',0,'correctionValues','green'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustGreen, self.jsonConfigBak, ['color','channelAdjustment',0,'pureRed','greenChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustGreen, self.jsonConfigBak, ['color','channelAdjustment',0,'pureGreen','greenChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustGreen, self.jsonConfigBak, ['color','channelAdjustment',0,'pureBlue','greenChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelGreen, self.jsonConfigBak, ['color','transform',0,'green','whitelevel'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelGreen, self.jsonConfigBak, ['color','transform',0,'green','blacklevel'],100)
				
				# blue
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaBlue, self.jsonConfigBak, ['color','transform',0,'blue','gamma'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdBlue, self.jsonConfigBak, ['color','transform',0,'blue','threshold'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureBlue, self.jsonConfigBak, ['color','temperature',0,'correctionValues','blue'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustBlue, self.jsonConfigBak, ['color','channelAdjustment',0,'pureRed','blueChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustBlue, self.jsonConfigBak, ['color','channelAdjustment',0,'pureGreen','blueChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustBlue, self.jsonConfigBak, ['color','channelAdjustment',0,'pureBlue','blueChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelBlue, self.jsonConfigBak, ['color','transform',0,'blue','whitelevel'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelBlue, self.jsonConfigBak, ['color','transform',0,'blue','blacklevel'],100)
				
				# device
				setConfigValueFromJson(config.plugins.hyperioncontrol.deviceType, self.jsonConfigBak, ['device','type'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.configName, self.jsonConfigBak, ['device','name'])
				outputIP = setConfigValueFromJson(config.plugins.hyperioncontrol.outputIP, self.jsonConfigBak, ['device','output'], onlyReturnValue=True)
				outputIP = outputIP.replace(":19446","").split(".")
				outputIP = [int(d) for d in outputIP]
				config.plugins.hyperioncontrol.outputIP.value = outputIP
				setConfigValueFromJson(config.plugins.hyperioncontrol.colorOrder, self.jsonConfigBak, ['device','colorOrder'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.baudrate, self.jsonConfigBak, ['device','rate'],calculate=1)
				
				#Framegrabber
				width = setConfigValueFromJson(None, self.jsonConfigBak, ['framegrabber','width'],None,True,"160")
				height = setConfigValueFromJson(None, self.jsonConfigBak, ['framegrabber','height'],None,True,"160")
				config.plugins.hyperioncontrol.framegrabberSize.value = str(width) + "," + str(height)
				setConfigValueFromJson(config.plugins.hyperioncontrol.framegrabberFreq, self.jsonConfigBak, ['framegrabber','frequency_Hz'],1)

				#BlackborderDetector(not in remote only in json)
				bbdEnable = setConfigValueFromJson(config.plugins.hyperioncontrol.bbdEnable, self.jsonConfigBak, ['blackborderdetector','enable'],calculate=None,onlyReturnValue=True)
				if str(bbdEnable).lower() == "true":
					config.plugins.hyperioncontrol.bbdEnable.value = True
				else:
					config.plugins.hyperioncontrol.bbdEnable.value = False
				setConfigValueFromJson(config.plugins.hyperioncontrol.bbdThreshold, self.jsonConfigBak, ['blackborderdetector','threshold'],calculate=100)
				
				#reload ConfigList with new values
				self.buildConfig()
			
			else:
				self.session.open(MessageBox,"Import fehlgeschlagen!\n\nKeine /etc/hyperion/hyperion.config.json.bak gefunden!", MessageBox.TYPE_INFO,timeout=5)
		except:
			import traceback
			traceback.print_exc()
			self.session.open(MessageBox,"Fehler beim Import der hyperion.config.json.bak!", MessageBox.TYPE_INFO,timeout=5)

	def readValuesFromConfig(self):
		
		try:
			#read values from config.json
			self.jsonConfig = ""
			config_file_path ="/etc/hyperion/hyperion.config.json"
			if os.path.exists(config_file_path): 
				valuesFromConfig = open(config_file_path).read()
				lines = valuesFromConfig.split("\n")
				newlines=[]
				for line in lines:
					cleanline = line.replace("\t","")
					if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
						newlines.append(line)
				valuesFromConfig = "\n".join(newlines)
				from collections import OrderedDict
				self.jsonConfig = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)
				
				#Framegrabber(not in remote only in json)
				config.plugins.hyperioncontrol.framegrabberSize.value = str(self.jsonConfig['framegrabber']['width']) + "," + str(self.jsonConfig['framegrabber']['height'])
				config.plugins.hyperioncontrol.framegrabberFreq.value = str(int(self.jsonConfig['framegrabber']['frequency_Hz']))
				#BlackborderDetector(not in remote only in json)
				bbdEnable = str(self.jsonConfig['blackborderdetector']['enable'])
				if bbdEnable == "true":
					config.plugins.hyperioncontrol.bbdEnable.value = True
				else:
					config.plugins.hyperioncontrol.bbdEnable.value = False
				config.plugins.hyperioncontrol.bbdThreshold.value = str(int(float(self.jsonConfig['blackborderdetector']['threshold']) * 100))
		except:
			import traceback
			traceback.print_exc()
			self.session.open(MessageBox,"Fehler beim Laden der /etc/hyperion/hyperion.config.json !", MessageBox.TYPE_INFO,timeout=5)

	def buildConfig(self):
		
		self.list = []
		if self.setupPage == "transform":
			self.list.append(getConfigListEntry(_("HSV (0-500, default=100)"), ))
			self.list.append(getConfigListEntry(_("Saturation Gain"), NoSave(config.plugins.hyperioncontrol.saturationGain)))
			self.list.append(getConfigListEntry(_("Value Gain"), NoSave(config.plugins.hyperioncontrol.valueGain)))
			self.list.append(getConfigListEntry("", )) 
			
			self.list.append(getConfigListEntry(_("HSL (0-100)"), ))
			self.list.append(getConfigListEntry(_("SaturationL Gain (default=100)"), NoSave(config.plugins.hyperioncontrol.saturationLGain)))
			self.list.append(getConfigListEntry(_("Luminance Gain (default=100)"), NoSave(config.plugins.hyperioncontrol.luminanceGain)))
			self.list.append(getConfigListEntry(_("Luminance Minimum (default=0)"), NoSave(config.plugins.hyperioncontrol.luminanceMinimum)))
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Gamma (0-500, default=250)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.gammaRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.gammaGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.gammaBlue)) )
			self.list.append(getConfigListEntry("", ))
					
			self.list.append(getConfigListEntry(_("Threshold (0-100, default=0)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.thresholdRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.thresholdGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.thresholdBlue)) )
			self.list.append(getConfigListEntry("", ))
					
			self.list.append(getConfigListEntry(_("Temperature (0-255, default=255)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.temperatureRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.temperatureGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.temperatureBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Red Adjust (0-255, default=255, 0, 0)"), ))
			self.list.append(getConfigListEntry(_("red (fix)"), NoSave(config.plugins.hyperioncontrol.redAdjustRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.redAdjustGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.redAdjustBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Green Adjust (0-255, default=0, 255, 0)"), ))
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.greenAdjustRed)) )
			self.list.append(getConfigListEntry(_("green (fix)"), NoSave(config.plugins.hyperioncontrol.greenAdjustGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.greenAdjustBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Blue Adjust (0-255, default=0, 0, 255)"), ))
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blueAdjustRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blueAdjustGreen)) )
			self.list.append(getConfigListEntry(_("blue (fix)"), NoSave(config.plugins.hyperioncontrol.blueAdjustBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Whitelevel (0-100, default=100)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.whitelevelRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.whitelevelGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.whitelevelBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Blacklevel (0-100, default=0)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blacklevelRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blacklevelGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.blacklevelBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Framegrabber"), ))
			self.list.append(getConfigListEntry(_("Size (default=160x160)"), NoSave(config.plugins.hyperioncontrol.framegrabberSize)) )
			self.list.append(getConfigListEntry(_("Frequency in Hz (10-25, default=10)"), NoSave(config.plugins.hyperioncontrol.framegrabberFreq)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("BlackBorderDetection"), ))
			self.list.append(getConfigListEntry(_("enabled"), NoSave(config.plugins.hyperioncontrol.bbdEnable)) )
			self.list.append(getConfigListEntry(_("threshold (1-20, default=2)"), NoSave(config.plugins.hyperioncontrol.bbdThreshold)) )
			
		elif self.setupPage == "device":
			self.list.append(getConfigListEntry(_("Device-Setup"), ))
			self.list.append(getConfigListEntry(_("Device-Typ"), config.plugins.hyperioncontrol.deviceType))
			self.list.append(getConfigListEntry(_("Config-Name"), config.plugins.hyperioncontrol.configName))
			self.list.append(getConfigListEntry(_("Device-IP (fix port: 19446)"), config.plugins.hyperioncontrol.outputIP))
			self.list.append(getConfigListEntry(_("colorOrder"), config.plugins.hyperioncontrol.colorOrder))
			self.list.append(getConfigListEntry(_("Baudrate"), config.plugins.hyperioncontrol.baudrate))
			
			self.list.append(getConfigListEntry("", ))
			self.list.append(getConfigListEntry(_("LED-Setup"), ))
			self.list.append(getConfigListEntry(_("LED-Anfang"), config.plugins.hyperioncontrol.ledBegin))
			if config.plugins.hyperioncontrol.ledBegin.value == "1":
				self.list.append(getConfigListEntry(_("LED-Richtung"), config.plugins.hyperioncontrol.ledDirection))
				self.list.append(getConfigListEntry(_("Anzahl LED oben"), config.plugins.hyperioncontrol.ledTop))
				self.list.append(getConfigListEntry(_("Anzahl LED rechts"), config.plugins.hyperioncontrol.ledRight))
				self.list.append(getConfigListEntry(_("Anzahl LED links"), config.plugins.hyperioncontrol.ledLeft))
				self.list.append(getConfigListEntry(_("Anzahl LED Lcke unten"), config.plugins.hyperioncontrol.ledBottomGap))
				self.list.append(getConfigListEntry(_("Bildbereich - Horizontale Tiefe (default=8%)"), config.plugins.hyperioncontrol.hPicDeep))
				self.list.append(getConfigListEntry(_("Bildbereich - Vertikale Tiefe (default=5%)"), config.plugins.hyperioncontrol.vPicDeep))
		
		self["config"].list = self.list
		self["config"].l.setList(self.list) 
	
	def getConfigName(self, subsection, configelement):
		for item in subsection.dict():
			conf = subsection.__getattr__(item)
			if conf == configelement:
				return item

	def changed(self):
		current = self["config"].getCurrent()
		configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])
		print "[HyperionControl] - change:", configName
		
		if configName == "ledBegin":
			self.buildConfig()
			return
		
		cmd = ""
		if configName == "saturationGain":
			value = int(config.plugins.hyperioncontrol.saturationGain.value) / 100.0
			cmd = "-s %s" % value
		elif configName == "valueGain":
			value = int(config.plugins.hyperioncontrol.valueGain.value) / 100.0
			cmd = "-v %s" % value
		elif configName == "saturationLGain":
			value = int(config.plugins.hyperioncontrol.saturationLGain.value) / 100.0
			cmd = "-u %s" % value
		elif configName == "luminanceGain":
			value = int(config.plugins.hyperioncontrol.luminanceGain.value) / 100.0
			cmd = "-m %s" % value
		elif configName == "luminanceMinimum":
			value = int(config.plugins.hyperioncontrol.luminanceMinimum.value) / 100.0
			cmd = "-n %s" % value
		elif configName.startswith("gamma"):
			Red = int(config.plugins.hyperioncontrol.gammaRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.gammaGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.gammaBlue.value) / 100.0
			cmd = "-g '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("threshold"):
			Red = int(config.plugins.hyperioncontrol.thresholdRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.thresholdGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.thresholdBlue.value) / 100.0
			cmd = "-t '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("temperature"):
			Red = int(config.plugins.hyperioncontrol.temperatureRed.value)
			Green = int(config.plugins.hyperioncontrol.temperatureGreen.value)
			Blue = int(config.plugins.hyperioncontrol.temperatureBlue.value)
			cmd = "-Z '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("redAdjust"):
			Red = int(config.plugins.hyperioncontrol.redAdjustRed.value)
			Green = int(config.plugins.hyperioncontrol.redAdjustGreen.value)
			Blue = int(config.plugins.hyperioncontrol.redAdjustBlue.value)
			cmd = "-R '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("greenAdjust"):
			Red = int(config.plugins.hyperioncontrol.greenAdjustRed.value)
			Green = int(config.plugins.hyperioncontrol.greenAdjustGreen.value)
			Blue = int(config.plugins.hyperioncontrol.greenAdjustBlue.value)
			cmd = "-G '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("blueAdjust"):
			Red = int(config.plugins.hyperioncontrol.blueAdjustRed.value)
			Green = int(config.plugins.hyperioncontrol.blueAdjustGreen.value)
			Blue = int(config.plugins.hyperioncontrol.blueAdjustBlue.value)
			cmd = "-B '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("whitelevel"):
			Red = int(config.plugins.hyperioncontrol.whitelevelRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.whitelevelGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.whitelevelBlue.value) / 100.0
			cmd = "-w '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("blacklevel"):
			Red = int(config.plugins.hyperioncontrol.blacklevelRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.blacklevelGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.blacklevelBlue.value) / 100.0
			cmd = "-b '%s %s %s'" % (Red, Green, Blue)
			
		if cmd:
			send_CMD(cmd)

	def key_red(self):
		config.plugins.hyperioncontrol.save()
		self.close()

	def key_green(self):
		self.saveJson()
		
	def key_yellow(self):
		if self.setupPage == "transform":
			self.setupPage = "device"
			self["key_yellow"].setText("Transform")
		else:
			self.setupPage = "transform"
			self["key_yellow"].setText("Device/LED")
		self.buildConfig()
	
	def key_blue(self):
		self.importJsonBak()
		pass
	
	def key_exit(self):
		config.plugins.hyperioncontrol.save()
		self.close(self.session, "exit")


class hyperionControlSystem(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlSystem" position="center,200" size="900,800" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,720" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,720" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,720"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,720" size="200,70" scale="stretch"/>
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,720" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="680,720" size="200,70" text="About" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="10,700" size="880,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="880,680" itemWidth="880" itemHeight="60" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionControlSystem" position="center,80" size="590,620" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="440,570" size="140,40" text="About" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="575,540" itemWidth="575" itemHeight="40" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""

	def __init__(self, session):
		Screen.__init__(self, session)
		
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions'], {
		'cancel': self.key_exit,
		'ok'	: self.key_ok,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		})

		self.litems = []
		tlf = TemplatedListFonts()
		self.ml = MenuList(self.litems, mode=eListbox.layoutGrid, content=eListboxPythonMultiContent, margin=ePoint(0,0), selectionZoom=1.3)
		self.ml.l.setFont(0, gFont(tlf.face(tlf.BIG), tlf.size(tlf.BIG)))
		self['list'] = self.ml	
		self.onLayoutFinish.append(self.buildList)
		self.setTitle("Hyperion Control - SystemCTL")

	def buildList(self):
		self.ml.l.setBuildFunc(self.showList, True)
		self.litems = []
		self.litems.append((("start hyperionaml", "start"),))
		self.litems.append((("stop hyperionaml", "stop"),))
		self.litems.append((("restart hyperionaml", "restart"),))
		self.litems.append((("status hyperionaml", "status"),))
		self.litems.append((("start hyperion", "start_1"),))
		self.litems.append((("stop hyperion", "stop_1"),))
		self.litems.append((("restart hyperion", "restart_1"),))
		self.litems.append((("status hyperion", "status_1"),))
		self.litems.append((("disable hyperion", "disable_1"),))
		self.litems.append((("enable autostart", "enable"),))
		self.litems.append((("disable autostart", "disable"),))
		self.litems.append((("status autostart", "status_2"),))
		self["list"].setList(self.litems)

	def showList(self, item, selected):
		res = [item]
		(name, cmd) = item
		if sz_w == 1920:
			res.append(MultiContentEntryText(pos=(0,0), size=(880,60), font=0, text=name, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		else:
			res.append(MultiContentEntryText(pos=(0,0), size=(575,40), font=0, text=name, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		return res

	def key_red(self):
		self.close()

	def key_green(self):
		pass

	def key_yellow(self):
		pass

	def key_blue(self):
		title="HyperionControl version %s \nplugin by\neinfall, pclin & Sven_H" % hyperioncontrol_version
		self.session.open(MessageBox,("%s") % (title),  MessageBox.TYPE_INFO)

	def key_exit(self):
		self.close(self.session, "exit")

	def key_ok(self):
		exist = self['list'].getCurrent()
		if exist == None:
			return
		(name, returnValue) = self['list'].getCurrent()[0]
		if returnValue == "start":
			self.session.open(Console,_("start hyperionaml"),["%s start hyperionaml" % hyperionremote_sh])
		elif returnValue == "stop":
			self.session.open(Console,_("stop hyperionaml"),["%s stop hyperionaml" % hyperionremote_sh])
		elif returnValue == "restart":
			self.session.open(Console,_("restart hyperionaml"),["%s restart hyperionaml" % hyperionremote_sh])
		elif returnValue == "status":
			self.session.open(Console,_("status hyperionaml"),["%s status hyperionaml" % hyperionremote_sh])
		if returnValue == "start_1":
			self.session.open(Console,_("start hyperion"),["%s start hyperion" % hyperionremote_sh])
		elif returnValue == "stop_1":
			self.session.open(Console,_("stop hyperion"),["%s stop hyperion" % hyperionremote_sh])
		elif returnValue == "restart_1":
			self.session.open(Console,_("restart hyperion"),["%s restart hyperion" % hyperionremote_sh])
		elif returnValue == "status_1":
			self.session.open(Console,_("status hyperion"),["%s status hyperion" % hyperionremote_sh])
		elif returnValue == "disable_1":
			self.session.open(Console,_("disable hyperion"),["%s disable hyperion" % hyperionremote_sh])
		elif returnValue == "enable":
			self.session.open(Console,_("enable systemd-hyperionaml.timer"),["%s enable systemd-hyperionaml.timer" % hyperionremote_sh])
		elif returnValue == "disable":
			self.session.open(Console,_("disable systemd-hyperionaml.timer"),["%s disable systemd-hyperionaml.timer" % hyperionremote_sh])
		elif returnValue == "status_2":
			self.session.open(Console,_("status systemd-hyperionaml.timer"),["%s status systemd-hyperionaml.timer" % hyperionremote_sh])

class hyperionControlEffects(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlEffects" position="center,200" size="900,800" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,720" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,720" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,720"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,720" size="200,70" scale="stretch"/>
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,720" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="10,700" size="880,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="880,680" itemWidth="880" itemHeight="60" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionControlEffects" position="center,80" size="590,620" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="575,540" itemWidth="575" itemHeight="40" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
		
	def __init__(self, session):
		Screen.__init__(self, session)
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', "ShortcutActions", "WizardActions"], {
		'cancel': self.key_exit,
		'ok'	: self.key_ok,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		'left'	: self.key_left,
		'right'	: self.key_right
		})
		self.setTitle("Hyperion Control - Effects")
		
		tlf = TemplatedListFonts()
		self.litems = [('Blue mood blobs',), ('Cinema brighten lights',), ('Cinema dim lights',), ('Cold mood blobs',), ('Color traces',), ('Full color mood blobs',), ('Green mood blobs',), ('Knight rider',), ('Police Lights Single',), ('Police Lights Solid',), ('Rainbow mood',), ('Rainbow swirl',), ('Rainbow swirl fast',), ('Random',), ('Red mood blobs',), ('Running dots',), ('Snake',), ('Sparks',), ('Sparks Color',), ('Strobe Raspbmc',), ('Strobe blue',), ('Strobe white',), ('Warm mood blobs',), ('X-Mas',), ('UDP listener',), ('UDP multicast listener',), ('System Shutdown',)]
		self.ml = MenuList(self.litems, mode=eListbox.layoutGrid, content=eListboxPythonMultiContent, margin=ePoint(0,0), selectionZoom=1.2)
		self.ml.l.setFont(0, gFont(tlf.face(tlf.BIG), tlf.size(tlf.BIG)))
		self['list'] = self.ml

		self.onLayoutFinish.append(self.buildList)

	def buildList(self):
		self.ml.l.setBuildFunc(self.showList, True)
		self["list"].setList(self.litems)

	def showList(self, item, selected):
		res = [item]
		if sz_w == 1920:
			res.append(MultiContentEntryText(pos=(0,0), size=(880,60), font=0, text=item, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		else:
			res.append(MultiContentEntryText(pos=(0,0), size=(575,40), font=0, text=item, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		return res

	def key_left(self):
		print "left"
		
	def key_right(self):
		print "right"

	def key_red(self):
		self.close()

	def key_green(self):
		pass
		
	def key_yellow(self):
		pass
		
	def key_blue(self):
		pass

	def key_ok(self):
		exist = self['list'].getCurrent()
		if exist == None:
			return
		effect = self['list'].getCurrent()[0]
		print effect
		cmd = "--effect '%s'" % effect
		send_CMD(cmd)

	def key_exit(self):
		self.close(self.session, "exit")


class hyperionStart(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionStart" position="center,center" size="900,650">
			<eLabel font="Regular;42" foregroundColor="white" name="" position="0,20" size="900,110" text="Hyperion Control" valign="center" halign="center" transparent="1" />
			<widget name="logo" position="80,170" size="250,250" />
			<widget name="status" position="420,170" size="450,250" font="Regular;23" foregroundColor="black" backgroundColor="white" />
			<eLabel name="" position="10,550" size="880,2" backgroundColor="grey" />
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,560" size="200,70" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,560" size="200,70" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,560" size="200,70" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,560" size="200,70" scale="stretch" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,560" size="200,70" text="Color" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="240,560" size="200,70" text="Effects" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="460,560" size="200,70" text="Setup " valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_blue" render="Label"  font="Regular;30" foregroundColor="white" position="680,560" size="200,70" valign="center" halign="center" zPosition="1" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="version" font="Regular;30" position="15,500" size="200,40" />
			<ePixmap pixmap="skin_default/buttons/key_menu.png" position="785,490" size="90,45" scale="stretch" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionStart" position="center,center" size="600,500">
			<eLabel font="Regular;32" foregroundColor="white" name="" position="0,20" size="600,98" text="Hyperion Control" valign="center" halign="center" transparent="1" />
			<widget name="logo" position="40,140" size="210,210" />
			<widget name="status" position="300,160" size="270,180" font="Regular;16" foregroundColor="black" backgroundColor="white" />
			<eLabel name="" position="5,440" size="590,2" backgroundColor="grey" />
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,450" size="135,40" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="160,450" size="135,40" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="300,450" size="135,40" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,450" size="135,40" scale="stretch" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="20,450" size="135,40" text="Color" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="160,450" size="135,40" text="Effects" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="300,450" size="135,40" text="Setup " valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;22" foregroundColor="white" position="440,450" size="135,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="version" font="Regular;22" position="15,400" size="200,25" />
			<ePixmap pixmap="skin_default/buttons/key_menu.png" position="497,395" size="72,36" />
		</screen>"""
	
	def __init__(self, session):
		Screen.__init__(self, session)
		
		self["logo"] = Pixmap()
		self["version"] = Label(_("Version") + " " + hyperioncontrol_version)
		self["status"] = Label("Status:")
		self["key_blue"] = StaticText("Live Mode")
		
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions'], 
		{
			'cancel': self.close,
			'red'	: self.key_red,
			'green'	: self.key_green,
			'yellow': self.key_yellow,
			'blue'	: self.key_blue,
			'menu': self.menu,
		})
		
		self.onLayoutFinish.append(self.layoutFinished)
		self.onShown.append(self.refreshStatus)
		self.runfirst = True

	def refreshStatus(self):
		status_txt = "\n  Status:\n\n"
		print "=== [HyperionControl] refresh status"
		
		#hyperion-status
		command = Command("systemctl status hyperion")
		command.run(timeout=3)
		cmd_out= command.out
		if "Active: active" in cmd_out:
			status_txt += "  Hyperion: active\n"
		else:
			status_txt += "  Hyperion: inactive\n"
		
		#hyperionaml-status
		command = Command("systemctl status hyperionaml")
		command.run(timeout=3)
		cmd_out= command.out
		if "Active: active" in cmd_out:
			status_txt += "  Hyperionaml: active\n"
		else:
			status_txt += "  Hyperionaml: inactive\n"
		
		#systemd-hyperionaml.timer
		command = Command("systemctl status systemd-hyperionaml.timer")
		command.run(timeout=3)
		cmd_out= command.out
		if "enabled;" in cmd_out:
			status_txt += "  Autostart: active\n\n"
		else:
			status_txt += "  Autostart: inactive\n\n"
		
		#Modus
		self.jsonRemote = readValuesFromRemote(self, self.runfirst, False)
		self.runfirst=False
		if self.jsonRemote:
			if self.jsonRemote["activeEffects"] != []:
				status_txt += "  Modus: Effects\n"
				#self["key_blue"].setText("Live Mode")
			elif self.jsonRemote["activeLedColor"] != []:
				status_txt += "  Modus: Color\n"
				#self["key_blue"].setText("Live Mode")
			else:
				status_txt += "  Modus: Live or Image\n"
				#self["key_blue"].setText("")
		
		self["status"].setText(status_txt)
	
	def layoutFinished(self):
		self["logo"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/hyperion_logo_rund.png")
		self.setTitle("Hyperion Control")

	def key_red(self):
		self.session.open(hyperionControlColors)

	def key_green(self):
		self.session.open(hyperionControlEffects)
		
	def key_yellow(self):
		self.session.open(hyperionControlSetup)

	def key_blue(self):
		if self["key_blue"].getText():
			send_CMD('--clearall')
			self.session.open(MessageBox,"Der Live Mode wurde aktiviert.", MessageBox.TYPE_INFO,timeout=5)
	
	def menu(self):
		self.session.open(hyperionControlSystem)

class HyperionControlPictureBrowser(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="HyperionControlPictureBrowser" position="center,170" size="1500,740" title="Hyperion Control Picture Browser">
			<ePixmap pixmap="Default-FHD/skin_default/buttons/red.svg" position="10,5" size="300,70" />
			<ePixmap pixmap="Default-FHD/skin_default/buttons/green.svg" position="310,5" size="300,70" />
			<widget backgroundColor="#9f1313" font="Regular;30" halign="center" position="10,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="300,70" source="key_red" transparent="1" valign="center" zPosition="1" />
			<widget backgroundColor="#1f771f" font="Regular;30" halign="center" position="310,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="300,70" source="key_green" transparent="1" valign="center" zPosition="1" />
			<eLabel backgroundColor="grey" position="10,80" size="900,1" />
			<eLabel backgroundColor="grey" position="920,5" size="1,725" />
			<widget name="picpreview" position="960,90" size="500,500" />
			<widget enableWrapAround="1" name="filelist" position="10,90" scrollbarMode="showOnDemand" size="900,630" />
		</screen>"""
	else:
		skin = """
		<screen name="HyperionControlPictureBrowser" position="center,120" size="920,430" title="Hyperion Control Picture Browser">
			<ePixmap pixmap="skin_default/buttons/red.png" position="10,5" size="200,40"/>
			<ePixmap pixmap="skin_default/buttons/green.png" position="210,5" size="200,40"/>
			<widget source="key_red" render="Label" position="10,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
			<widget source="key_green" render="Label" position="210,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
			<eLabel position="10,50" size="590,1" backgroundColor="grey"/>
			<eLabel position="610,5" size="1,420" backgroundColor="grey"/>
			<widget name="picpreview" position="620,80" size="290,290" />
			<widget name="filelist" position="10,60" size="590,360" enableWrapAround="1" scrollbarMode="showOnDemand"/>
		</screen>"""

	def __init__(self, session, current_path=None):
		Screen.__init__(self, session)
		if current_path:
			currDir = current_path
			if not os.path.exists(currDir):
				currDir = "/"
		else:
			currDir = None #start with mountpoints-list

		self.filelist = FileList(currDir, matchingPattern="(?i)^.*\.(jpg|png)")
		self["filelist"] = self.filelist
		self["picpreview"] = Pixmap()
		self["filelist"].onSelectionChanged.append(self.selectionChanged)
		
		self["FilelistActions"] = ActionMap(["SetupActions"],
			{
				"save": self.ok,
				"ok": self.ok,
				"cancel": self.exit
			})
		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("OK"))
		
		self.ThumbTimer = eTimer()
		self.ThumbTimer_conn = self.ThumbTimer.timeout.connect(self.showThumb)
		
		self.onShown.append(self.setWindowTitle)

	def setWindowTitle(self):
		self.setTitle(_("Hyperion Control Picture Browser"))

	def ok(self):
		if self["filelist"].canDescent(): # isDir
			self["filelist"].descent()
		else:
			file_name = self["filelist"].getFilename()
			path = self["filelist"].getCurrentDirectory()
			self.exit(path + file_name)

	def showThumb(self):
		if not self["filelist"].canDescent():
			file_name = self["filelist"].getFilename()
			path = self["filelist"].getCurrentDirectory()
			print "== picname", path + file_name
			self["picpreview"].instance.setPixmapFromFile(path + file_name)
			self["picpreview"].show()

	def selectionChanged(self):
		if self["picpreview"] is None:
			return
		
		if not self["filelist"].canDescent():
			self.ThumbTimer.start(400, True)
		else:
			self["picpreview"].hide()

	def exit(self, retvalue=None):
		self.ThumbTimer.stop()
		self.ThumbTimer_conn = None
		if self.filelist.getCurrentDirectory() is None:
			config.plugins.hyperioncontrol.lastpicpath.value = "/"
		else:
			config.plugins.hyperioncontrol.lastpicpath.value = self["filelist"].getCurrentDirectory()
		config.plugins.hyperioncontrol.save()
		
		self["picpreview"] = None
		self.close(retvalue)


def send_CMD(cmd):
	raw = "hyperion-remote %s" % cmd
	print "[HyperionControl] SEND CMD TO Hyperion Remote: %s" % raw
	os.system(raw)

def setConfigValueFromJson(config, jsonData, jsonKey, calculate=None, onlyReturnValue=False, default=None):
	try:
		for item in jsonKey:
			jsonData = jsonData[item]
		if calculate:
			jsonData = int(jsonData * calculate)
		if onlyReturnValue:
			print "=== [HyperionControl] return value", item, str(jsonData)
			return str(jsonData)
		else:
			config.value = str(jsonData)
			print "=== [HyperionControl] set value to", item, str(jsonData)
	except (KeyError, TypeError, IndexError):
		if onlyReturnValue:
			print "=== [HyperionControl] error on item - return default", item, config.default
			if default:
				return default
			else:
				return config.default
		else:
			config.value = config.default
			print "=== [HyperionControl] error on item - set default", item, config.default

def readValuesFromRemote(self, showMessage=True, setConfig=True):
	print "=== [HyperionControl] readValuesFromRemote"
	try:
		command = Command("hyperion-remote -l")
		command.run(timeout=5)
		
		#read real settings from hyperion-remote -l
		if command.timeout:
			print "[HyperionControl] timeout on readValuesFromRemote"
			if showMessage:
				self.session.open(MessageBox,"Timeout beim Laden der remote-Daten!\n\n", MessageBox.TYPE_INFO,timeout=5)
			return None
		elif command.process.returncode != 0:
			print "[HyperionControl] error on readValuesFromRemote:", str(command.process.returncode), str(command.err)
			if showMessage:
				self.session.open(MessageBox,"Fehler beim Laden der remote-Daten!\n\n" + str(command.err), MessageBox.TYPE_INFO,timeout=5)
			return None
		
		valuesFromRemote = command.out
		
		pos = valuesFromRemote.find("Server info:")
		if pos:
			valuesFromRemote = valuesFromRemote[pos+12:]
			jsonRemote = json.loads(valuesFromRemote)
			if setConfig:
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationGain, jsonRemote, ['transform',0,'saturationGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.valueGain, jsonRemote, ['transform',0,'valueGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationLGain, jsonRemote, ['transform',0,'saturationLGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceGain, jsonRemote, ['transform',0,'luminanceGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceMinimum, jsonRemote, ['transform',0,'luminanceMinimum'],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaRed, jsonRemote, ['transform',0,'gamma',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaGreen, jsonRemote, ['transform',0,'gamma',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaBlue, jsonRemote, ['transform',0,'gamma',2],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdRed, jsonRemote, ['transform',0,'threshold',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdGreen, jsonRemote, ['transform',0,'threshold',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdBlue, jsonRemote, ['transform',0,'threshold',2],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustRed, jsonRemote, ['adjustment',0,'redAdjust',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustGreen, jsonRemote, ['adjustment',0,'redAdjust',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustBlue, jsonRemote, ['adjustment',0,'redAdjust',2])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustRed, jsonRemote, ['adjustment',0,'greenAdjust',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustGreen, jsonRemote, ['adjustment',0,'greenAdjust',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustBlue, jsonRemote, ['adjustment',0,'greenAdjust',2])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustRed, jsonRemote, ['adjustment',0,'blueAdjust',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustGreen, jsonRemote, ['adjustment',0,'blueAdjust',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustBlue, jsonRemote, ['adjustment',0,'blueAdjust',2])
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureRed, jsonRemote, ['temperature',0,'correctionValues',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureGreen, jsonRemote, ['temperature',0,'correctionValues',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureBlue, jsonRemote, ['temperature',0,'correctionValues',2])
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelRed, jsonRemote, ['transform',0,'blacklevel',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelGreen, jsonRemote, ['transform',0,'blacklevel',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelBlue, jsonRemote, ['transform',0,'blacklevel',2],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelRed, jsonRemote, ['transform',0,'whitelevel',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelGreen, jsonRemote, ['transform',0,'whitelevel',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelBlue, jsonRemote, ['transform',0,'whitelevel',2],100)
			
			return jsonRemote

	except:
		import traceback, sys
		traceback.print_exc()
		exc_type, exc_value, exc_traceback = sys.exc_info()
		if showMessage:
			error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
			self.session.open(MessageBox,"Fehler beim Laden der remote-Daten!\n\nFehler:\n%s" % error, MessageBox.TYPE_INFO,timeout=10)
		return None

